@@ -10,7 +10,7 @@ use nextest_metadata::NextestExitCode;
 use nextest_runner::{errors::*, helpers::plural, redact::Redactor};
 use owo_colors::OwoColorize;
 use semver::Version;
-use std::{error::Error, path::PathBuf, process::ExitStatus, string::FromUtf8Error};
+use std::{error::Error, io, path::PathBuf, process::ExitStatus, string::FromUtf8Error};
 use swrite::{SWrite, swriteln};
 use thiserror::Error;
 use tracing::{Level, error, info};
@@ -107,6 +107,12 @@ pub enum ExpectedError {
         #[from]
         err: TargetTripleError,
     },
+    #[error("remap absolute error")]
+    RemapAbsoluteError {
+        arg_name: &'static str,
+        path: Utf8PathBuf,
+        error: io::Error,
+    },
     #[error("metadata materialize error")]
     MetadataMaterializeError {
         arg_name: &'static str,
@@ -413,6 +419,7 @@ impl ExpectedError {
             | Self::TestFilterBuilderError { .. }
             | Self::HostPlatformDetectError { .. }
             | Self::TargetTripleError { .. }
+            | Self::RemapAbsoluteError { .. }
             | Self::MetadataMaterializeError { .. }
             | Self::UnknownArchiveFormat { .. }
             | Self::ArchiveExtractError { .. }
@@ -743,6 +750,18 @@ impl ExpectedError {
                     err.source()
                 }
             }
+            Self::RemapAbsoluteError {
+                arg_name,
+                path,
+                error,
+            } => {
+                error!(
+                    "error making {} path absolute: {}",
+                    arg_name.style(styles.bold),
+                    path.style(styles.bold),
+                );
+                Some(error as &dyn Error)
+            }
             Self::MetadataMaterializeError { arg_name, err } => {
                 error!(
                     "error reading metadata from argument {}",
@@ -947,7 +966,7 @@ impl ExpectedError {
             } => {
                 let current_dir_str = match current_dir {
                     Ok(dir) => dir.to_string_lossy().into_owned(),
-                    Err(e) => format!("(error: {})", e),
+                    Err(e) => format!("(error: {e})"),
                 };
                 error!(
                     "[double-spawn] failed to exec `{command:?}`, current_dir: `{current_dir_str}`"
