@@ -28,6 +28,7 @@ use indexmap::IndexMap;
 use nextest_filtering::{
     BinaryQuery, EvalContext, Filterset, FiltersetKind, ParseContext, TestQuery,
 };
+use quick_junit::ReportUuid;
 use serde::{Deserialize, de::Error};
 use smol_str::SmolStr;
 use std::{
@@ -36,6 +37,7 @@ use std::{
     process::Command,
     sync::Arc,
 };
+use swrite::{SWrite, swrite};
 
 /// The scripts defined in nextest configuration.
 #[derive(Clone, Debug, Default, Deserialize)]
@@ -548,6 +550,16 @@ impl ScriptId {
         &self.0
     }
 
+    /// Returns a unique ID for this script, consisting of the run ID, the script ID, and the stress index.
+    pub fn unique_id(&self, run_id: ReportUuid, stress_index: Option<u32>) -> String {
+        let mut out = String::new();
+        swrite!(out, "{run_id}:{self}");
+        if let Some(stress_index) = stress_index {
+            swrite!(out, "@stress-{}", stress_index);
+        }
+        out
+    }
+
     #[cfg(test)]
     pub(super) fn as_str(&self) -> &str {
         self.0.as_str()
@@ -1200,7 +1212,7 @@ mod tests {
         indoc! {r#"
             [scripts.setup.foo]
         "#},
-        "scripts.setup.foo: missing field `command`"
+        r#"scripts.setup.foo: missing configuration field "scripts.setup.foo.command""#
 
         ; "missing command"
     )]
@@ -1227,7 +1239,7 @@ mod tests {
             [scripts.setup.foo]
             command = { relative-to = "target" }
         "#},
-        "missing field `command-line`"
+        r#"missing configuration field "scripts.setup.foo.command.command-line""#
 
         ; "missing command-line in table"
     )]
