@@ -35,6 +35,8 @@ use nextest_metadata::{
     RustTestSuiteSummary, TestListSummary,
 };
 use owo_colors::OwoColorize;
+use quick_junit::ReportUuid;
+use serde::Serialize;
 use std::{
     borrow::Cow,
     collections::{BTreeMap, BTreeSet},
@@ -43,6 +45,7 @@ use std::{
     path::PathBuf,
     sync::{Arc, OnceLock},
 };
+use swrite::{SWrite, swrite};
 use tokio::runtime::Runtime;
 use tracing::debug;
 
@@ -1243,7 +1246,7 @@ impl<'a> TestCommandCli<'a> {
 /// A key for identifying and sorting test instances.
 ///
 /// Returned by [`TestInstance::id`].
-#[derive(Clone, Copy, Debug, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq, Ord, PartialOrd, Serialize)]
 pub struct TestInstanceId<'a> {
     /// The binary ID.
     pub binary_id: &'a RustBinaryId,
@@ -1252,6 +1255,30 @@ pub struct TestInstanceId<'a> {
     pub test_name: &'a str,
 }
 
+impl TestInstanceId<'_> {
+    /// Return the attempt ID corresponding to this test instance.
+    ///
+    /// This string uniquely identifies a single test attempt.
+    pub fn attempt_id(
+        &self,
+        run_id: ReportUuid,
+        stress_index: Option<u32>,
+        attempt: u32,
+    ) -> String {
+        let mut out = String::new();
+        swrite!(out, "{run_id}:{}", self.binary_id);
+        if let Some(stress_index) = stress_index {
+            swrite!(out, "@stress-{}", stress_index);
+        }
+        swrite!(out, "${}", self.test_name);
+        if attempt > 1 {
+            swrite!(out, "#{attempt}");
+        }
+
+        out
+    }
+}
+
 impl fmt::Display for TestInstanceId<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{} {}", self.binary_id, self.test_name)
