@@ -972,10 +972,10 @@ pub struct SetupScriptEnvMap {
 #[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord)]
 pub struct RetryData {
     /// The current attempt. In the range `[1, total_attempts]`.
-    pub attempt: usize,
+    pub attempt: u32,
 
     /// The total number of times this test can be run. Equal to `1 + retries`.
-    pub total_attempts: usize,
+    pub total_attempts: u32,
 }
 
 impl RetryData {
@@ -1034,6 +1034,17 @@ impl ExecutionResult {
             | ExecutionResult::Timeout => false,
         }
     }
+
+    /// Returns a static string representation of the result.
+    pub fn as_static_str(&self) -> &'static str {
+        match self {
+            ExecutionResult::Pass => "pass",
+            ExecutionResult::Leak { .. } => "leak",
+            ExecutionResult::Fail { .. } => "fail",
+            ExecutionResult::ExecFail => "exec-fail",
+            ExecutionResult::Timeout => "timeout",
+        }
+    }
 }
 
 /// Failure status: either an exit code or an abort status.
