@@ -22,7 +22,7 @@ use crate::{
     },
     runner::{ExecutorEvent, RunUnitQuery, SignalRequest, StressCondition, StressCount},
     signal::{JobControlEvent, ShutdownEvent, SignalEvent, SignalHandler, SignalInfoEvent},
-    time::StopwatchStart,
+    time::{StopwatchSnapshot, StopwatchStart},
 };
 use chrono::Local;
 use debug_ignore::DebugIgnore;
@@ -437,6 +437,7 @@ where
                     }
                 }
                 self.new_setup_script(script_id.clone(), config, index, total, req_tx);
+
                 self.callback_none_response(TestEventKind::SetupScriptStarted {
                     stress_index,
                     index,
@@ -454,14 +455,29 @@ where
                 program,
                 elapsed,
                 will_terminate,
-            }) => self.callback_none_response(TestEventKind::SetupScriptSlow {
-                stress_index,
-                script_id,
-                program,
-                args: &config.command.args,
-                elapsed,
-                will_terminate: will_terminate.is_some(),
-            }),
+            }) => {
+                // Fire the USDT probe for setup script slow.
+                crate::fire_usdt!(UsdtSetupScriptSlow {
+                    id: script_id.unique_id(self.run_id, stress_index.map(|s| s.current)),
+                    run_id: self.run_id,
+                    script_id: script_id.to_string(),
+                    program: program.clone(),
+                    args: config.command.args.clone(),
+                    elapsed_nanos: elapsed.as_nanos() as u64,
+                    will_terminate: will_terminate.is_some(),
+                    stress_current: stress_index.map(|s| s.current),
+                    stress_total: stress_index.and_then(|s| s.total.map(|t| t.get())),
+                });
+
+                self.callback_none_response(TestEventKind::SetupScriptSlow {
+                    stress_index,
+                    script_id,
+                    program,
+                    args: &config.command.args,
+                    elapsed,
+                    will_terminate: will_terminate.is_some(),
+                })
+            }
             InternalEvent::Executor(ExecutorEvent::SetupScriptFinished {
                 stress_index,
                 script_id,
@@ -473,6 +489,30 @@ where
             }) => {
                 self.finish_setup_script();
                 self.run_stats.on_setup_script_finished(&status);
+
+                // Fire the setup-script-done probe, extracting the exit code
+                // from the result if available.
+                let _exit_code = match status.result {
+                    crate::reporter::events::ExecutionResult::Fail {
+                        failure_status: crate::reporter::events::FailureStatus::ExitCode(code),
+                        ..
+                    } => Some(code),
+                    _ => None,
+                };
+
+                crate::fire_usdt!(UsdtSetupScriptDone {
+                    id: script_id.unique_id(self.run_id, stress_index.map(|s| s.current)),
+                    run_id: self.run_id,
+                    script_id: script_id.to_string(),
+                    program: program.clone(),
+                    args: config.command.args.clone(),
+                    result: status.result.as_static_str(),
+                    exit_code: _exit_code,
+                    duration_nanos: status.time_taken.as_nanos() as u64,
+                    stress_current: stress_index.map(|s| s.current),
+                    stress_total: stress_index.and_then(|s| s.total.map(|t| t.get())),
+                });
+
                 // Setup scripts failing always cause the entire test run to be cancelled
                 // (--no-fail-fast is ignored).
                 let fail_cancel = !status.result.is_success();
@@ -529,13 +569,33 @@ where
                 retry_data,
                 elapsed,
                 will_terminate,
-            }) => self.callback_none_response(TestEventKind::TestSlow {
-                stress_index,
-                test_instance,
-                retry_data,
-                elapsed,
-                will_terminate: will_terminate.is_some(),
-            }),
+            }) => {
+                // Fire the test-slow probe.
+                crate::fire_usdt!(UsdtTestAttemptSlow {
+                    attempt_id: test_instance.id().attempt_id(
+                        self.run_id,
+                        stress_index.map(|s| s.current),
+                        retry_data.attempt,
+                    ),
+                    run_id: self.run_id,
+                    binary_id: test_instance.suite_info.binary_id.clone(),
+                    test_name: test_instance.name.to_owned(),
+                    attempt: retry_data.attempt,
+                    total_attempts: retry_data.total_attempts,
+                    elapsed_nanos: elapsed.as_nanos() as u64,
+                    will_terminate: will_terminate.is_some(),
+                    stress_current: stress_index.map(|s| s.current),
+                    stress_total: stress_index.and_then(|s| s.total.map(|t| t.get())),
+                });
+
+                self.callback_none_response(TestEventKind::TestSlow {
+                    stress_index,
+                    test_instance,
+                    retry_data,
+                    elapsed,
+                    will_terminate: will_terminate.is_some(),
+                })
+            }
             InternalEvent::Executor(ExecutorEvent::AttemptFailedWillRetry {
                 stress_index,
                 test_instance,
@@ -877,7 +937,7 @@ where
         }
     }
 
-    pub(super) fn run_finished(&mut self) {
+    pub(super) fn run_finished(&mut self) -> StopwatchSnapshot {
         let stopwatch_end = self.stopwatch.snapshot();
         self.basic_callback(TestEventKind::RunFinished {
             start_time: stopwatch_end.start_time.fixed_offset(),
@@ -890,7 +950,8 @@ where
                     || RunFinishedStats::Single(self.run_stats),
                     RunFinishedStats::Stress,
                 ),
-        })
+        });
+        stopwatch_end
     }
 
     pub(super) fn run_stats(&self) -> RunStats {
