@@ -224,6 +224,7 @@ impl<'a> ExecutorContext<'a> {
 
         let mut attempt = 0;
         let mut delay = Duration::ZERO;
+
         let last_run_status = loop {
             attempt += 1;
             let retry_data = RetryData {
@@ -388,6 +389,20 @@ impl<'a> ExecutorContext<'a> {
             .id()
             .expect("child has never been polled so must return a PID");
 
+        // Fire the USDT probe for setup script start.
+        crate::fire_usdt!(UsdtSetupScriptStart {
+            id: script
+                .script_id
+                .unique_id(self.run_id, script.stress_index.map(|s| s.current)),
+            run_id: self.run_id,
+            script_id: script.script_id.to_string(),
+            pid: child_pid,
+            program: script.program.clone(),
+            args: script.config.command.args.clone(),
+            stress_current: script.stress_index.map(|s| s.current),
+            stress_total: script.stress_index.and_then(|s| s.total.map(|t| t.get())),
+        });
+
         // If assigning the child to the job fails, ignore this. This can happen if the process has
         // exited.
         let _ = super::os::assign_process_to_job(&child, job.as_ref());
@@ -415,7 +430,7 @@ impl<'a> ExecutorContext<'a> {
             slow_after: None,
         };
 
-        let (res, leaked) = {
+        let (res, leak_info) = {
             let res = loop {
                 tokio::select! {
                     () = child_acc.fill_buf(), if !child_acc.fds.is_done() => {}
@@ -546,7 +561,7 @@ impl<'a> ExecutorContext<'a> {
                 })
             });
 
-            let leaked = detect_fd_leaks(
+            let leak_info = detect_fd_leaks(
                 &cx,
                 child_pid,
                 &mut child_acc,
@@ -557,7 +572,7 @@ impl<'a> ExecutorContext<'a> {
             )
             .await;
 
-            (res, leaked)
+            (res, leak_info)
         };
 
         let exit_status = match res {
@@ -570,6 +585,7 @@ impl<'a> ExecutorContext<'a> {
 
         let exit_status = exit_status.expect("None always results in early return");
 
+        let leaked = matches!(leak_info, LeakDetectInfo::Leaked);
         let exec_result = status.unwrap_or_else(|| {
             create_execution_result(exit_status, &child_acc.errors, leaked, leak_timeout.result)
         });
@@ -645,6 +661,7 @@ impl<'a> ExecutorContext<'a> {
             test.settings.run_wrapper(),
             test.settings.run_extra_args(),
         );
+
         let command_mut = cmd.command_mut();
 
         // Debug environment variable for testing.
@@ -691,6 +708,10 @@ impl<'a> ExecutorContext<'a> {
         // best-effort thing.
         let job = super::os::create_job().ok();
 
+        // Capture program and args before spawn moves cmd
+        let _program = cmd.program().to_owned();
+        let _args = cmd.args().to_owned();
+
         let crate::test_command::Child {
             mut child,
             child_fds,
@@ -706,6 +727,24 @@ impl<'a> ExecutorContext<'a> {
             .id()
             .expect("child has never been polled so must return a PID");
 
+        crate::fire_usdt!(UsdtTestAttemptStart {
+            attempt_id: test.test_instance.id().attempt_id(
+                self.run_id,
+                test.stress_index.map(|s| s.current),
+                test.retry_data.attempt,
+            ),
+            run_id: self.run_id,
+            binary_id: test.test_instance.suite_info.binary_id.clone(),
+            test_name: test.test_instance.name.to_owned(),
+            pid: child_pid,
+            program: _program,
+            args: _args,
+            attempt: test.retry_data.attempt,
+            total_attempts: test.retry_data.total_attempts,
+            stress_current: test.stress_index.map(|s| s.current),
+            stress_total: test.stress_index.and_then(|s| s.total.map(|t| t.get())),
+        });
+
         // If assigning the child to the job fails, ignore this. This can happen if the process has
         // exited.
         let _ = super::os::assign_process_to_job(&child, job.as_ref());
@@ -727,7 +766,7 @@ impl<'a> ExecutorContext<'a> {
             slow_after: None,
         };
 
-        let (res, leaked) = {
+        let (res, leak_info) = {
             let res = loop {
                 tokio::select! {
                     () = child_acc.fill_buf(), if !child_acc.fds.is_done() => {}
@@ -857,7 +896,7 @@ impl<'a> ExecutorContext<'a> {
                 })
             });
 
-            let leaked = detect_fd_leaks(
+            let leak_info = detect_fd_leaks(
                 &cx,
                 child_pid,
                 &mut child_acc,
@@ -868,7 +907,7 @@ impl<'a> ExecutorContext<'a> {
             )
             .await;
 
-            (res, leaked)
+            (res, leak_info)
         };
 
         let exit_status = match res {
@@ -880,10 +919,39 @@ impl<'a> ExecutorContext<'a> {
         };
 
         let exit_status = exit_status.expect("None always results in early return");
+
+        let (leaked, _time_to_close) = match leak_info {
+            LeakDetectInfo::NoLeak { time_to_close } => (false, Some(time_to_close)),
+            LeakDetectInfo::Leaked => (true, None),
+        };
+
         let exec_result = status.unwrap_or_else(|| {
             create_execution_result(exit_status, &child_acc.errors, leaked, leak_timeout.result)
         });
 
+        let stopwatch_end = stopwatch.snapshot();
+
+        // Fire the test-attempt-done probe
+        crate::fire_usdt!(UsdtTestAttemptDone {
+            attempt_id: test.test_instance.id().attempt_id(
+                self.run_id,
+                test.stress_index.map(|s| s.current),
+                test.retry_data.attempt,
+            ),
+            run_id: self.run_id,
+            binary_id: test.test_instance.suite_info.binary_id.clone(),
+            test_name: test.test_instance.name.to_owned(),
+            attempt: test.retry_data.attempt,
+            total_attempts: test.retry_data.total_attempts,
+            result: exec_result.as_static_str(),
+            exit_code: exit_status.code(),
+            duration_nanos: stopwatch_end.active.as_nanos() as u64,
+            leaked,
+            time_to_close_fds_nanos: _time_to_close.map(|d| d.as_nanos() as u64),
+            stress_current: test.stress_index.map(|s| s.current),
+            stress_total: test.stress_index.and_then(|s| s.total.map(|t| t.get())),
+        });
+
         Ok(InternalExecuteStatus {
             test,
             slow_after: cx.slow_after,
@@ -893,7 +961,7 @@ impl<'a> ExecutorContext<'a> {
                 errors: ErrorList::new(UnitKind::WAITING_ON_TEST_MESSAGE, child_acc.errors),
             },
             result: exec_result,
-            stopwatch_end: stopwatch.snapshot(),
+            stopwatch_end,
         })
     }
 }
@@ -902,7 +970,7 @@ impl<'a> ExecutorContext<'a> {
 struct BackoffIter {
     policy: RetryPolicy,
     current_factor: f64,
-    remaining_attempts: usize,
+    remaining_attempts: u32,
 }
 
 impl BackoffIter {
@@ -1205,6 +1273,18 @@ async fn handle_delay_between_attempts<'a>(
     }
 }
 
+/// Information about file descriptor leak detection.
+#[derive(Debug, Clone, Copy)]
+enum LeakDetectInfo {
+    /// No leak detected. File descriptors closed within the timeout period.
+    NoLeak {
+        /// Time taken for file descriptors to close.
+        time_to_close: Duration,
+    },
+    /// Leak detected. File descriptors did not close before timeout.
+    Leaked,
+}
+
 /// After a child process has exited, detect if it leaked file handles by
 /// leaving long-running grandchildren open.
 ///
@@ -1220,7 +1300,7 @@ async fn detect_fd_leaks<'a>(
     leak_timeout: LeakTimeout,
     stopwatch: &mut StopwatchStart,
     req_rx: &mut UnboundedReceiver<RunUnitRequest<'a>>,
-) -> bool {
+) -> LeakDetectInfo {
     loop {
         // Ignore stop and continue events here since the leak timeout should be very small.
         // TODO: we may want to consider them.
@@ -1228,12 +1308,14 @@ async fn detect_fd_leaks<'a>(
         let waiting_stopwatch = crate::time::stopwatch();
 
         tokio::select! {
+            biased;
+
             // All of the branches here need to check for
             // `!child_acc.fds.is_done()`, because if child_fds is done we want
             // to hit the `else` block right away.
             () = child_acc.fill_buf(), if !child_acc.fds.is_done() => {}
             () = &mut sleep, if !child_acc.fds.is_done() => {
-                break true;
+                break LeakDetectInfo::Leaked;
             }
             recv = req_rx.recv(), if !child_acc.fds.is_done() => {
                 // The sender stays open longer than the whole loop, and the
@@ -1274,7 +1356,8 @@ async fn detect_fd_leaks<'a>(
                 }
             }
             else => {
-                break false;
+                let time_to_close = waiting_stopwatch.snapshot().active;
+                break LeakDetectInfo::NoLeak { time_to_close };
             }
         }
     }
